﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TechNote11_DPSerialPort_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private const int SEND_DATA_SIZE = 300;

        private string m_SendDatas;
        private string m_strTemp = "";
        private uint m_iTotalCount = 0;
        private uint m_iFailCount = 0;
        private uint m_iResponseCount = 0;

        private string[] m_strPortNo = { "COM1", "COM2", "COM3", "COM4", "COM7" };
        private string[] m_strBaudRate = { "115200", "57600", "38400", "19200", "14400", "9600", "4800", "2400", "1200", "600", "300", "110" };

        private void Form1_Load(object sender, EventArgs e)
        {
            smartForm1.MainForm = this;

            cmbPortNo.DataSource = m_strPortNo;
            cmbPortNo.SelectItemIndex = 0;

            cmbBaudRate.DataSource = m_strBaudRate;
            cmbBaudRate.SelectItemIndex = 0;

            smartSerialPort1.STXCode = 0x02;
            smartSerialPort1.ETXCode = 0x03;
            smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.NONE;
            smartSerialPort1.ErrorCode_Location = SmartX.SmartSerialPort.ERRORCODELOCATION.HEADER;
            smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.STXANDETX;
            smartSerialPort1.ReceiveDetect = SmartX.SmartSerialPort.RECEIVEDETECTTYPE.EVENT_QUEUE;
            smartSerialPort1.FrameBufferSize = 500;
            smartSerialPort1.ReadTimeout = 1000;

            lsbData.ItemAddOrder = SmartX.SmartListBox.ITEMADDORDERS.TOPADD;

            m_SendDatas = GetDataGenerate(SEND_DATA_SIZE);
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (smartSerialPort1.IsOpen == true)
            {
                smartSerialPort1.Close();
            }
            smartTimer1.Stop();
        }

        // Server Message Receive 시작 버튼
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (btnStart.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
            {
                // SmartComboBox에서 선택한 Item의 Index로 Baud_Rate 속성값을 설정
                smartSerialPort1.Baud_Rate = (SmartX.SmartSerialPort.BAUDRATE)Convert.ToInt32(cmbBaudRate.Text);

                // SmartComboBox에서 선택한 Item의 Index로 PortNo 속성값을 설정
                if (cmbPortNo.SelectItemIndex == 4)
                {
                    smartSerialPort1.PortNo = (SmartX.SmartSerialPort.COMPORTNO)(cmbPortNo.SelectItemIndex + 3);
                }
                else
                {
                    smartSerialPort1.PortNo = (SmartX.SmartSerialPort.COMPORTNO)(cmbPortNo.SelectItemIndex + 1);
                }

                smartSerialPort1.Open();

                smartTimer1.Start();
            }
            else
            {
                if (smartSerialPort1.IsOpen == true)
                {
                    smartSerialPort1.Close();
                }
                smartTimer1.Stop();
            }
        }

        // Program 종료 버튼
        private void btnExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            // 수신된 프레임 데이터를 표시 합니다.
            ReceiveFrameDataDisplay();
        }

        private string GetDataGenerate(int iSize)
        {
            string retStr = "";
            int iCnt = 0;

            while (true)
            {
                for (char i = 'A'; i <= 'Z'; i++)
                {
                    retStr += i.ToString();
                    if (++iCnt >= iSize)
                    {
                        return retStr;
                    }
                }
            }
        }

        // 수신된 프레임 단위 데이터를 표시한다.
        private void ReceiveFrameDataDisplay()
        {
            if (m_iTotalCount % 1000 == 0)
            {
                lsbData.ClearAll();
            }

            SmartX.SmartSerialPort.FRAMEDATAREADSTATUS iRetCode = SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY;
            iRetCode = smartSerialPort1.ReadQueue(out m_strTemp);

            ++m_iTotalCount;
            switch (iRetCode)
            {
                case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY:
                    break;

                case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.FAILDATA:
                    ++m_iFailCount;
                    break;

                // Client로부터 정상 Data가 입력됨
                case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.VALIDDATA:
                    // 입력된 Data가 "HNS_SMARTX"이면
                    if (m_strTemp == "HNS_SMARTX")
                    {
                        ++m_iResponseCount;
                        lsbData.AddItem("[" + m_iResponseCount.ToString() + "]" + m_strTemp);

                        // Client로 전송
                        smartSerialPort1.WriteFrame(m_SendDatas, SmartX.SmartSerialPort.CODETYPES.UNICODE);
                    }
                    else
                    {
                        ++m_iFailCount;
                    }
                    break;
            }
        }
    }
}