﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TechNote11_OverrunTest_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private const int SEND_DATA_SIZE = 300;

        private uint m_iTotalCount = 0;
        private uint m_iFailCount = 0;
        private uint m_iSuccessCount = 0;

        private string m_strSendCommand = "HNS_SMARTX";
        private string m_ReceiveOrgDatas;
        private string m_ReceiveDatas;

        private string[] m_strPortNo = { "COM1", "COM2", "COM3", "COM4", "COM7" };
        private string[] m_strBaudRate = { "115200", "57600", "38400", "19200", "14400", "9600", "4800", "2400", "1200", "600", "300", "110" };

        private void Form1_Load(object sender, EventArgs e)
        {
            smartForm1.MainForm = this;

            cmbPortNo.DataSource = m_strPortNo;
            cmbPortNo.SelectItemIndex = 0;

            cmbBaudRate.DataSource = m_strBaudRate;
            cmbBaudRate.SelectItemIndex = 0;

            smartSerialPort1.STXCode = 0x02;
            smartSerialPort1.ETXCode = 0x03;
            smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.NONE;
            smartSerialPort1.ErrorCode_Location = SmartX.SmartSerialPort.ERRORCODELOCATION.HEADER;
            smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.STXANDETX;
            smartSerialPort1.ReceiveDetect = SmartX.SmartSerialPort.RECEIVEDETECTTYPE.EVENT_QUEUE;
            smartSerialPort1.FrameBufferSize = 5000;
            smartSerialPort1.ReadTimeout = 1000;

            m_ReceiveOrgDatas = GetDataGenerate(SEND_DATA_SIZE);

            lsbError.ItemAddOrder = SmartX.SmartListBox.ITEMADDORDERS.TOPADD;
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (smartSerialPort1.IsOpen == true)
            {
                smartSerialPort1.Close();
            }
            smartTimer1.Stop();
        }

        // Client Message Send 시작 버튼
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (btnStart.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
            {
                // SmartComboBox에서 선택한 Item의 Index로 Baud_Rate 속성값을 설정
                smartSerialPort1.Baud_Rate = (SmartX.SmartSerialPort.BAUDRATE)Convert.ToInt32(cmbBaudRate.Text);

                // SmartComboBox에서 선택한 Item의 Index로 PortNo 속성값을 설정
                if (cmbPortNo.SelectItemIndex == 4)
                {
                    smartSerialPort1.PortNo = (SmartX.SmartSerialPort.COMPORTNO)(cmbPortNo.SelectItemIndex + 3);
                }
                else
                {
                    smartSerialPort1.PortNo = (SmartX.SmartSerialPort.COMPORTNO)(cmbPortNo.SelectItemIndex + 1);
                }

                smartSerialPort1.Open();

                // 대량의 데이터를 보내도록 명령 전송 처리
                smartSerialPort1.WriteFrame(m_strSendCommand, SmartX.SmartSerialPort.CODETYPES.UNICODE);

                smartTimer1.Start();
            }
            else
            {
                if (smartSerialPort1.IsOpen == true)
                {
                    smartSerialPort1.Close();
                }
                smartTimer1.Stop();
            }
        }

        // Program 종료 버튼
        private void btnExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            // 수신된 프레임 데이터를 표시 합니다.
            smartTimer1.Stop();
            ReceiveFrameDataDisplay();
            smartTimer1.Start();
        }

        private void smartSerialPort1_OnPortError(System.IO.Ports.SerialError eErrorState)
        {
            lsbError.AddItem(eErrorState.ToString());
        }

        private string GetDataGenerate(int iSize)
        {
            string retStr = "";
            int iCnt = 0;

            while (true)
            {
                for (char i = 'A'; i <= 'Z'; i++)
                {
                    retStr += i.ToString();
                    if (++iCnt >= iSize)
                    {
                        return retStr;
                    }
                }
            }
        }

        // 수신된 프레임 단위 데이터를 표시 한다.
        private void ReceiveFrameDataDisplay()
        {
            if (m_iTotalCount % 1000 == 0)
            {
                lsbData.ClearAll();
            }

            SmartX.SmartSerialPort.FRAMEDATAREADSTATUS iRetCode = SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY;
            iRetCode = smartSerialPort1.ReadQueue(out m_ReceiveDatas);

            if (m_iTotalCount < 900000000)
            {
                switch (iRetCode)
                {
                    case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY:
                        System.Threading.Thread.Sleep(500);
                        // Buffer Overrun Error가 발생할 경우 데이터 수신이 안됩니다.
                        // 따라서 일정시간 대기했다가 다시 수신 데이터를 확인합니다.
                        iRetCode = smartSerialPort1.ReadQueue(out m_ReceiveDatas);
                        switch (iRetCode)
                        {
                            case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY:
                            case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.FAILDATA:
                                ++m_iFailCount;
                                break;

                            // Sever로부터 정상 Data가 입력됨
                            case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.VALIDDATA:
                                if (m_ReceiveDatas == m_ReceiveOrgDatas)
                                {
                                    lsbData.AddItem(m_ReceiveDatas);
                                    ++m_iSuccessCount;
                                }
                                else
                                {
                                    ++m_iFailCount;
                                }
                                break;
                        }
                        break;

                    case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.FAILDATA:
                        ++m_iFailCount;
                        break;

                    // Sever로부터 정상 Data가 입력됨
                    case SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.VALIDDATA:
                        if (m_ReceiveDatas == m_ReceiveOrgDatas)
                        {
                            lsbData.AddItem(m_ReceiveDatas);
                            ++m_iSuccessCount;
                        }
                        else
                        {
                            ++m_iFailCount;
                        }
                        break;
                }

                ++m_iTotalCount;

                labTotalCount.Text = m_iTotalCount.ToString();
                labFailCount.Text = m_iFailCount.ToString();
                labSuccessCount.Text = m_iSuccessCount.ToString();

                if (smartSerialPort1.IsOpen == true)
                {
                    // 대량의 데이터를 보내도록 명령 전송 처리
                    smartSerialPort1.WriteFrame(m_strSendCommand, SmartX.SmartSerialPort.CODETYPES.UNICODE);
                }
            }
        }
    }
}